@@ -1,13 +1,28 @@
-/* $Id: flock.c,v 1.9 2005/07/10 07:10:58 hpa Exp $ */
+/* $Id: flock.c,v 1.13 2005/07/11 05:32:56 hpa Exp $ */
 /* ----------------------------------------------------------------------- *
  *   
  *   Copyright 2003-2005 H. Peter Anvin - All Rights Reserved
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation, Inc., 53 Temple Place Ste 330,
- *   Bostom MA 02111-1307, USA; either version 2 of the License, or
- *   (at your option) any later version; incorporated herein by reference.
+ *   Permission is hereby granted, free of charge, to any person
+ *   obtaining a copy of this software and associated documentation
+ *   files (the "Software"), to deal in the Software without
+ *   restriction, including without limitation the rights to use,
+ *   copy, modify, merge, publish, distribute, sublicense, and/or
+ *   sell copies of the Software, and to permit persons to whom
+ *   the Software is furnished to do so, subject to the following
+ *   conditions:
+ *   
+ *   The above copyright notice and this permission notice shall
+ *   be included in all copies or substantial portions of the Software.
+ *   
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ *   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ *   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ *   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ *   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ *   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
  *
  * ----------------------------------------------------------------------- */
 
@@ -47,7 +62,7 @@ static void usage(int ex)
   fprintf(stderr,
 	  "flock version " VERSION "\n"
 	  "Usage: %s [-seun][-w #] fd#\n"
-	  "       %s [-sen][-w #] file [-c] command...\n"
+	  "       %s [-seon][-w #] file [-c] command...\n"
 	  "  -s  --shared     Get a shared lock\n"
 	  "  -e  --exclusive  Get an exclusive lock\n"
 	  "  -u  --unlock     Remove a lock\n"
@@ -126,7 +141,8 @@ int main(int argc, char *argv[])
   
   memset(&timeout, 0, sizeof timeout);
 
-  while ( (opt = getopt_long(argc, argv, "+sexnouw:h?", long_options, &ix)) != EOF ) {
+  optopt = 0;
+  while ( (opt = getopt_long(argc, argv, "+sexnouw:hV?", long_options, &ix)) != EOF ) {
     switch(opt) {
     case 's':
       type = LOCK_SH;
@@ -151,10 +167,11 @@ int main(int argc, char *argv[])
 	usage(EX_USAGE);
       break;
     case 'V':
-      fprintf(stderr, "flock " VERSION "\n");
+      printf("flock " VERSION "\n");
       exit(0);
     default:
-      usage((optopt != 'h' && optopt != '?') ? EX_USAGE : 0);
+      /* optopt will be set if this was an unrecognized option, i.e. *not* 'h' or '?' */
+      usage(optopt ? EX_USAGE : 0);
       break;
     }
   }
@@ -210,7 +227,8 @@ int main(int argc, char *argv[])
   if ( have_timeout ) {
     if ( timeout.it_value.tv_sec == 0 &&
 	 timeout.it_value.tv_usec == 0 ) {
-      /* -w 0 is equivalent to -n */
+      /* -w 0 is equivalent to -n; this has to be special-cased
+	 because setting an itimer to zero means disabled! */
 
       have_timeout = 0;
       block = LOCK_NB;
@@ -231,7 +249,7 @@ int main(int argc, char *argv[])
       exit(1);
     case EINTR:			/* Signal received */
       if ( timeout_expired )
-	return 1;		/* -w option set and failed to lock */
+	exit(1);		/* -w option set and failed to lock */
       continue;			/* otherwise try again */
     default:			/* Other errors */
       if ( filename )
